#property copyright "MetaQuotes Corporation"
#property version   "1.00"
#property description "RMI = Relative Momentum Index"
#property indicator_separate_window

//+-------------------------------------------------------------------+
//|                                       Relative Momentum Index.mq4 |
//|                              Original indicator by davinviproject |
//|                  Modified and Corrected by Linuxser for forex TSD |
//|                                                                   |
//| Relative Momentum Index (RMI)                                     |
//| Overview                                                          |
//| The Relative Momentum Index (RMI) was developed by Roger Altman.  |
//| It was first introduced in the February 1993 issue of Technical   |
//| Analysis of Stocks & Commodities magazine.                        |
//| The RMI is a variation of the RSI indicator. The RMI counts up and|
//| down days from the close relative to the close x-days ago (where x|
//| is not limited to 1 as is required by the RSI) instead of counting|
//| up and down days from close to close as the RSI does.             |
//|                                                                   |
//| Note that an RMI with parameters of C, 14, 1 is equivalent to a 14|
//| period RSI of the Close price. This is because the momentum       |
//|parameter is calculating only a 1-day price change (which the RSI  |
//|does by (default).                                                 |
//|As the momentum periods are increased the RMI fluctuations become  |
//|smoother.                                                          |
//| Since the RMI is an oscillator it exhibits the same strengths and |
//| weaknesses of other overbought / oversold indicators.             |
//|During strong trending markets it is likely that the RMI will      | 
//|remain at overbought or oversold levels for an extended period of  |
//|time.                                                              |  
//|However, during non-trending markets the RMI tends to oscillate more 
//|predictably between an overbought level of 70 to 90 and an oversold|
//|level of 10 to 30.                                                 |
//+-------------------------------------------------------------------+

#property indicator_buffers 4
#property indicator_plots 2

#property indicator_minimum 0
#property indicator_maximum 100

#property indicator_levelstyle STYLE_SOLID
#property indicator_levelwidth 1
#property indicator_levelcolor clrDarkOliveGreen
#property indicator_level1 30
#property indicator_level2 70
#property indicator_level3 50

#property indicator_label1 "RMI Main"
#property indicator_color1 clrAqua
#property indicator_type1  DRAW_LINE
#property indicator_style1 STYLE_SOLID
#property indicator_width1 2

#property indicator_label2  "RMI Signal"
#property indicator_color2  clrLavender
#property indicator_type2   DRAW_LINE
#property indicator_style2  STYLE_DOT
#property indicator_width2  1

extern int RMIPeriod = 14;
extern int MomPeriod = 5;
extern int RMIMAPeriod = 9;

double RMIBuffer[];
double RMISignalBuff[];
double PosBuffer[];
double NegBuffer[];

//+------------------------------------------------------------------+
int OnInit() {
     
   string short_name;
   
   SetIndexBuffer(0,RMIBuffer,INDICATOR_DATA);
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,RMIPeriod);
   SetIndexBuffer(1,RMISignalBuff,INDICATOR_DATA);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,RMIPeriod);
   SetIndexBuffer(2,PosBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,NegBuffer,INDICATOR_CALCULATIONS); 
   
   short_name = "RMI-MA ("+IntegerToString(RMIPeriod)+","+IntegerToString(MomPeriod)+","+IntegerToString(RMIMAPeriod)+")";
   IndicatorSetString(INDICATOR_SHORTNAME,short_name);
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);  

   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]) {

   if(rates_total<RMIPeriod) return(0);
   if(prev_calculated<0 || prev_calculated>rates_total) return(0);
   
   int startbar; //This variable refers to the rightmost bar that is not yet complete.
   if(prev_calculated==0) { startbar = RMIPeriod; }
   else startbar = prev_calculated-1;    
 
   for(int bar = startbar; bar < rates_total; bar++ ) { //Recalculation is done on every tick.
      double diff = close[bar] - close[bar-MomPeriod];      
      PosBuffer[bar] = (PosBuffer[bar-1] * (RMIPeriod-1) + (diff > 0.0 ? diff  : 0.0)) / RMIPeriod;
      NegBuffer[bar] = (NegBuffer[bar-1] * (RMIPeriod-1) + (diff < 0.0 ? -diff : 0.0)) / RMIPeriod;
      
      if ( NegBuffer[bar] != 0.0 ) RMIBuffer[bar] = 100.0 - 100.0/(1 + PosBuffer[bar] / NegBuffer[bar]); 
      else RMIBuffer[bar] = 50.0;
      RMISignalBuff[bar] = iMAOnArrayMQL4(RMIBuffer,rates_total-1,RMIMAPeriod,0,MODE_EMA,bar);
   }
   return(rates_total);
}

//+------------------------------------------------------------------+
double iMAOnArrayMQL4(double &array[],
                      int total,
                      int period,
                      int ma_shift,
                      int ma_method,
                      int shift) {
                      
   double buf[], arr[];
   if(total==0) total = ArraySize(array);
   if(total>0 && total<=period) return(0);
   if(shift>total-period-ma_shift) return(0);
   
   switch(ma_method) {
      case MODE_SMA: {
         total=ArrayCopy(arr,array,0,shift+ma_shift,period);
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0; int i, pos=total-1;
         for(i=1; i<period; i++, pos--) sum += arr[pos];
         while(pos>=0) {
            sum += arr[pos];
            buf[pos] = sum/period;
            sum -= arr[pos+period-1];
            pos--;
         }
         return(buf[0]);
      }
      case MODE_EMA: {
         if(ArrayResize(buf,total)<0) return(0);
         double pr = 2.0/(period+1);
         int pos = total-2;
         while(pos>=0) {
            if(pos==total-2) buf[pos+1]=array[pos+1];
            buf[pos]=array[pos]*pr+buf[pos+1]*(1-pr);
            pos--;
         }
         return(buf[shift+ma_shift]);
      }
      case MODE_SMMA: {
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0; int i, k, pos;
         pos=total-period;
         while(pos>=0) {
            if(pos==total-period) {
               for(i=0, k=pos; i<period; i++, k++) {
                  sum += array[k];
                  buf[k] = 0;
               }
            }
            else sum = buf[pos+1]*(period-1)+array[pos];
            buf[pos] = sum/period;
            pos--;
         }
         return(buf[shift+ma_shift]);
      }
      case MODE_LWMA: {
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0.0,lsum=0.0; double price;
         int i, weight=0, pos=total-1;
         for(i=1; i<=period; i++, pos--) {
            price = array[pos];
            sum += price*i;
            lsum += price;
            weight += i;
         }
         pos++;
         i = pos+period;
         while(pos>=0) {
            buf[pos] = sum/weight;
            if(pos==0) break;
            pos--;
            i--;
            price = array[pos];
            sum = sum-lsum+price*period;
            lsum -= array[i];
            lsum += price;
         }
         return(buf[shift+ma_shift]);
      }
      default: return(0);
   }
   return(0);
} 